#ifndef _devopen_h_
#define _devopen_h_

struct devinfo {
    int p_ctrl;
    int p_remote;
    int p_ram;
    int p_dsp;
    volatile u_int32_t* base_ctrl;
    volatile u_int32_t* base_remote;
    u_int32_t size_ctrl;
    u_int32_t size_remote;
    void (*close)(struct devinfo*);
};

int open_dev(char** pathes, struct devinfo*);

#endif
