cmd_/home/luster/sis3100/V2.02/dev/pci/sis1100.o := ld -m elf_i386  -r -o /home/luster/sis3100/V2.02/dev/pci/sis1100.o /home/luster/sis3100/V2.02/dev/pci/f1_rem_init.o /home/luster/sis3100/V2.02/dev/pci/pcisupport_linux2.4.o /home/luster/sis3100/V2.02/dev/pci/plx9054_reset.o /home/luster/sis3100/V2.02/dev/pci/sis1100_autoconf_linux.o /home/luster/sis3100/V2.02/dev/pci/sis1100_dsp.o /home/luster/sis3100/V2.02/dev/pci/sis1100_eeprom.o /home/luster/sis3100/V2.02/dev/pci/sis1100_front_io.o /home/luster/sis3100/V2.02/dev/pci/sis1100_init.o /home/luster/sis3100/V2.02/dev/pci/sis1100_init_remote.o /home/luster/sis3100/V2.02/dev/pci/sis1100_init_sdram.o /home/luster/sis3100/V2.02/dev/pci/sis1100_ioctl.o /home/luster/sis3100/V2.02/dev/pci/sis1100_irq.o /home/luster/sis3100/V2.02/dev/pci/sis1100_irq_handler.o /home/luster/sis3100/V2.02/dev/pci/sis1100_irq_thread.o /home/luster/sis3100/V2.02/dev/pci/sis1100_lemo_handler.o /home/luster/sis3100/V2.02/dev/pci/sis1100_llseek_linux.o /home/luster/sis3100/V2.02/dev/pci/sis1100_mbx0_handler.o /home/luster/sis3100/V2.02/dev/pci/sis1100_mmap.o /home/luster/sis3100/V2.02/dev/pci/sis1100.mod.o /home/luster/sis3100/V2.02/dev/pci/sis1100_open.o /home/luster/sis3100/V2.02/dev/pci/sis1100_pipe_linux.o /home/luster/sis3100/V2.02/dev/pci/sis1100_poll.o /home/luster/sis3100/V2.02/dev/pci/sis1100_read_block.o /home/luster/sis3100/V2.02/dev/pci/sis1100_read.o /home/luster/sis3100/V2.02/dev/pci/sis1100_read_dma.o /home/luster/sis3100/V2.02/dev/pci/sis1100_read_dma_linux.o /home/luster/sis3100/V2.02/dev/pci/sis1100_read_loop.o /home/luster/sis3100/V2.02/dev/pci/sis1100rem_front_io.o /home/luster/sis3100/V2.02/dev/pci/sis1100rem_init.o /home/luster/sis3100/V2.02/dev/pci/sis1100_sgl_map_user_linux.o /home/luster/sis3100/V2.02/dev/pci/sis1100_synch_handler.o /home/luster/sis3100/V2.02/dev/pci/sis1100_tmp_read.o /home/luster/sis3100/V2.02/dev/pci/sis1100_tmp_write.o /home/luster/sis3100/V2.02/dev/pci/sis1100_write_block.o /home/luster/sis3100/V2.02/dev/pci/sis1100_write.o /home/luster/sis3100/V2.02/dev/pci/sis1100_write_dma.o /home/luster/sis3100/V2.02/dev/pci/sis1100_write_dma_linux.o /home/luster/sis3100/V2.02/dev/pci/sis1100_write_loop.o /home/luster/sis3100/V2.02/dev/pci/sis1100_write_pipe.o /home/luster/sis3100/V2.02/dev/pci/sis3100rem_front_io.o /home/luster/sis3100/V2.02/dev/pci/sis3100rem_init.o /home/luster/sis3100/V2.02/dev/pci/sis3100rem_irq.o /home/luster/sis3100/V2.02/dev/pci/sis5100rem_front_io.o /home/luster/sis3100/V2.02/dev/pci/sis5100rem_init.o /home/luster/sis3100/V2.02/dev/pci/sis5100rem_irq.o /home/luster/sis3100/V2.02/dev/pci/vertex_rem_init.o
