/* $ZEL: sis1100_write_block.c,v 1.2 2004/05/27 23:10:38 wuestner Exp $ */

/*
 * Copyright (c) 2003-2004
 * 	Peter Wuestner.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sis1100_sc.h"

#if !defined(__NetBSD__) && ! defined(__linux__)
#error Invalid or Unknown Operating System
#endif

int
sis1100_write_block(struct sis1100_softc* sc, struct sis1100_fdata* fd,
        int size, int fifo, size_t num, size_t* num_written, int space,
        int32_t am, u_int32_t addr, const u_int8_t* data, u_int32_t* error)
{
    if (!num) {*error=0; *num_written=0; return 0;}
    if (sc->remote_hw==sis1100_hw_invalid) return ENXIO;

    if (!ACCESS_OK((u_int8_t*)data, num*size, 1)) return EFAULT;

    if (num==1) {
        u_int32_t val;
        switch (size) {
#ifdef __NetBSD__
            case 1: val=fubyte(data); break;
            case 2: val=fusword(data); break;
            case 4: val=fuword(data); break;
#elif __linux__
            case 1: __get_user(val, (u_int8_t*)data); break;
            case 2: __get_user(val, (u_int16_t*)data); break;
            case 4: __get_user(val, (u_int32_t*)data); break;
#endif
            default: val=0;
        }
        *error=sis1100_tmp_write(sc, addr, am, size, space, val);
        *num_written=1;
        return 0;
    } else {
        int res;
        if ((size==4)&&fd->mindmalen_w&&(num>=fd->mindmalen_w)) {
            res=sis1100_write_dma(sc, fd, addr, am, size,
                    space, fifo, num, num_written, data, error);
        } else {
            res=sis1100_write_loop(sc, fd, addr, am, size,
                    space, fifo, num, num_written, data, error);
        }
        return res;
    }
}
