#include <linux/module.h>
#include <linux/vermagic.h>
#include <linux/compiler.h>

MODULE_INFO(vermagic, VERMAGIC_STRING);

#undef unix
struct module __this_module
__attribute__((section(".gnu.linkonce.this_module"))) = {
 .name = __stringify(KBUILD_MODNAME),
 .init = init_module,
#ifdef CONFIG_MODULE_UNLOAD
 .exit = cleanup_module,
#endif
};

static const struct modversion_info ____versions[]
__attribute_used__
__attribute__((section("__versions"))) = {
	{ 0x65b4e613, "struct_module" },
	{ 0x39681cfa, "device_remove_file" },
	{ 0x7da8156e, "__kmalloc" },
	{ 0x87d59f9b, "complete_and_exit" },
	{ 0x43ca7336, "mem_map" },
	{ 0x1683aa15, "del_timer" },
	{ 0x9c72b530, "autoremove_wake_function" },
	{ 0x6c3397fb, "malloc_sizes" },
	{ 0xb02b2495, "pci_release_regions" },
	{ 0xd16ac615, "__get_user_1" },
	{ 0xda02d67, "jiffies" },
	{ 0x9925ce9c, "__might_sleep" },
	{ 0xd7474566, "__copy_to_user_ll" },
	{ 0x9ef719e7, "remap_page_range" },
	{ 0x80eca12a, "wait_for_completion" },
	{ 0xc2669443, "pci_set_master" },
	{ 0x1af40e18, "__copy_from_user_ll" },
	{ 0x863d23a3, "pci_set_dma_mask" },
	{ 0x1b7d4074, "printk" },
	{ 0x5d1d2e36, "rwsem_wake" },
	{ 0x1075bf0, "panic" },
	{ 0x9774eaa5, "dma_free_coherent" },
	{ 0x3425df38, "pci_bus_write_config_dword" },
	{ 0x17136545, "mod_timer" },
	{ 0xe1e0a7ea, "dma_alloc_coherent" },
	{ 0xe8d95495, "flush_signals" },
	{ 0xcad363f, "device_create_file" },
	{ 0x49e79940, "__cond_resched" },
	{ 0x123d3b6a, "kmem_cache_alloc" },
	{ 0x3762cb6e, "ioremap_nocache" },
	{ 0xbf758c44, "pci_bus_read_config_dword" },
	{ 0xe68b49b8, "get_user_pages" },
	{ 0x26e96637, "request_irq" },
	{ 0x4292364c, "schedule" },
	{ 0xfb6af58d, "recalc_sigpending" },
	{ 0x6506d836, "register_chrdev" },
	{ 0x6d172fe6, "pci_unregister_driver" },
	{ 0xab821cad, "__wake_up" },
	{ 0x37a0cba, "kfree" },
	{ 0x932da67e, "kill_proc" },
	{ 0x2e60bace, "memcpy" },
	{ 0xd4bde4d6, "pci_request_regions" },
	{ 0xe419f205, "prepare_to_wait" },
	{ 0xedc03953, "iounmap" },
	{ 0xc192d491, "unregister_chrdev" },
	{ 0xc3d60c67, "put_page" },
	{ 0x1105e73c, "finish_wait" },
	{ 0x7e9ebb05, "kernel_thread" },
	{ 0xd22b546, "__up_wakeup" },
	{ 0x4888a014, "__get_user_2" },
	{ 0x25da070, "snprintf" },
	{ 0xf2520b76, "__down_failed" },
	{ 0x7a93221b, "pci_enable_device" },
	{ 0x18076ad, "pci_set_consistent_dma_mask" },
	{ 0xdc43a9c8, "daemonize" },
	{ 0xd347765c, "rwsem_down_read_failed" },
	{ 0xf20dabd8, "free_irq" },
	{ 0xaaa30118, "pci_register_driver" },
};

static const char __module_depends[]
__attribute_used__
__attribute__((section(".modinfo"))) =
"depends=";

