cmd_/home/luster/sis3100/V2.02/dev/pci/sis1100.ko := ld -m elf_i386 -r -o /home/luster/sis3100/V2.02/dev/pci/sis1100.ko /home/luster/sis3100/V2.02/dev/pci/sis1100.o /home/luster/sis3100/V2.02/dev/pci/sis1100.mod.o
