/*
 * $ZEL$
 */

#include "test_3100.h"

static int
check_RESET(struct path* path)
{
    if (path->type!=sis1100_subdev_remote) return 0;
#ifdef SIS1100_REMOTE_RESET
    if (ioctl(path->p, SIS1100_REMOTE_RESET, &path->mapsize)<0) {
        printf("ioctl(%s, SIS3100_RESET): %s\n",
                path->name, strerror(errno));
        return -1;
    }
#else
    if (ioctl(path->p, SIS3100_RESET, &path->mapsize)<0) {
        printf("ioctl(%s, SIS3100_RESET): %s\n",
                path->name, strerror(errno));
        return -1;
    }
#endif
    return 0;
}
