/*
 * $ZEL: plx_eeprom.h,v 1.1 2003/08/21 17:30:34 wuestner Exp $
 */

#ifndef _plx_eeprom_h_
#define _plx_eeprom_h_

#include <stdio.h>

#define EEPROM_LEN 0x2c

extern char* eeprom_names[0x2c];

int eeprom_read(int p, u_int16_t* data, int start, int len);
void eeprom_dump(u_int16_t* data, int start, int len, int v);
int read_eepom_data_from_file(FILE* f, u_int16_t* data, u_int16_t* addr, int* len);
int eeprom_write(int p, u_int16_t* data, int start, int len);

#endif
