/* $ZEL: sis1100_lemo_handler.c,v 1.3 2004/05/27 23:10:24 wuestner Exp $ */

/*
 * Copyright (c) 2003-2004
 * 	Peter Wuestner.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "sis1100_sc.h"

void
sis1100_lemo_handler(struct sis1100_softc* sc)
{
    DECLARE_SPINLOCKFLAGS(s)

/*
 *     pINFO(sc, "LEMO_Handler called");
 *     pINFO(sc, "lemo_status=0x%08x", sc->lemo_status);
 */
    SPIN_LOCK_IRQSAVE(sc->lock_lemo_status, s);
    sc->new_irqs|=sc->lemo_status;
    sc->pending_irqs|=sc->lemo_status;
    sc->lemo_status=0;
    SPIN_UNLOCK_IRQRESTORE(sc->lock_lemo_status, s);
/*
 *     pINFO(sc, "sc->pending_irqs=0x%08x sc->new_irqs=0x%08x",
 *         sc->pending_irqs, sc->new_irqs);
 */
}
